@@ -1,5 +1,5 @@
-from typing import Dict, Tuple
 from decimal import Decimal, DecimalException
+from typing import Dict, Tuple, Optional
 
 from django.utils.translation import gettext as _
 from django.core.validators import URLValidator
@@ -39,7 +39,7 @@ logger = getLogger(__name__)
 @renderer_classes([JSONRenderer, BrowsableAPIRenderer])
 @parser_classes([MultiPartParser, FormParser, JSONParser])
 @validate_sep10_token()
-def deposit(account: str, request: Request) -> Response:
+def deposit(account: str, client_domain: Optional[str], request: Request,) -> Response:
     args = parse_request_args(request)
     if "error" in args:
         return args["error"]
@@ -61,6 +61,7 @@ def deposit(account: str, request: Request) -> Response:
         ),
         claimable_balance_supported=args["claimable_balance_supported"],
         on_change_callback=args["on_change_callback"],
+        client_domain=client_domain,
     )
 
     try:
@@ -74,7 +75,8 @@ def deposit(account: str, request: Request) -> Response:
         response, status_code = validate_response(
             args, integration_response, transaction
         )
-    except (ValueError, KeyError):
+    except (ValueError, KeyError) as e:
+        logger.error(str(e))
         return render_error_response(
             _("unable to process the request"), status_code=500
         )
@@ -104,14 +106,45 @@ def validate_response(
     if not (
         "how" in integration_response and isinstance(integration_response["how"], str)
     ):
-        logger.error("Invalid 'how' returned from process_sep6_request()")
-        raise ValueError()
+        raise ValueError("Invalid 'how' returned from process_sep6_request()")
     response = {
-        "min_amount": round(asset.deposit_min_amount, asset.significant_decimals),
-        "max_amount": round(asset.deposit_max_amount, asset.significant_decimals),
         "how": integration_response["how"],
         "id": transaction.id,
     }
+    if "min_amount" in integration_response:
+        if type(integration_response["min_amount"]) not in [Decimal, int, float]:
+            raise ValueError(
+                "Invalid 'min_amount' type returned from process_sep6_request()"
+            )
+        elif integration_response["min_amount"] < 0:
+            raise ValueError(
+                "Invalid 'min_amount' returned from process_sep6_request()"
+            )
+        response["min_amount"] = integration_response["min_amount"]
+    elif (
+        transaction.asset.deposit_min_amount
+        > Asset._meta.get_field("deposit_min_amount").default
+    ):
+        response["min_amount"] = round(
+            transaction.asset.deposit_min_amount, transaction.asset.significant_decimals
+        )
+    if "max_amount" in integration_response:
+        if type(integration_response["max_amount"]) not in [Decimal, int, float]:
+            raise ValueError(
+                "Invalid 'max_amount' type returned from process_sep6_request()"
+            )
+        elif integration_response["max_amount"] < 0:
+            raise ValueError(
+                "Invalid 'max_amount' returned from process_sep6_request()"
+            )
+        response["max_amount"] = integration_response["max_amount"]
+    elif (
+        transaction.asset.deposit_max_amount
+        < Asset._meta.get_field("deposit_max_amount").default
+    ):
+        response["max_amount"] = round(
+            transaction.asset.deposit_max_amount, transaction.asset.significant_decimals
+        )
 
     if calculate_fee == registered_fee_func:
         # Polaris user has not replaced default fee function, so fee_fixed
@@ -124,8 +157,9 @@ def validate_response(
     if "extra_info" in integration_response:
         response["extra_info"] = integration_response["extra_info"]
         if not isinstance(response["extra_info"], dict):
-            logger.error("Invalid 'extra_info' returned from process_sep6_request()")
-            raise ValueError()
+            raise ValueError(
+                "Invalid 'extra_info' returned from process_sep6_request()"
+            )
 
     return response, 200
 
