@@ -1,4 +1,6 @@
 """This module defines the logic for the `/transaction` endpoint."""
+from typing import Optional
+
 from django.utils.translation import gettext as _
 from django.views.decorators.clickjacking import xframe_options_exempt
 from rest_framework.decorators import api_view, renderer_classes
@@ -29,7 +31,9 @@ def more_info(request: Request) -> Response:
 @api_view(["GET"])
 @renderer_classes([JSONRenderer, BrowsableAPIRenderer])
 @validate_sep10_token()
-def transactions(account: str, request: Request) -> Response:
+def transactions(
+    account: str, _client_domain: Optional[str], request: Request,
+) -> Response:
     """
     Definition of the /transactions endpoint, in accordance with SEP-0006.
     See: https://github.com/stellar/stellar-protocol/blob/master/ecosystem/sep-0006.md#transaction-history
@@ -45,7 +49,9 @@ def transactions(account: str, request: Request) -> Response:
 @api_view(["GET"])
 @renderer_classes([JSONRenderer, BrowsableAPIRenderer])
 @validate_sep10_token()
-def transaction(account: str, request: Request) -> Response:
+def transaction(
+    account: str, _client_domain: Optional[str], request: Request,
+) -> Response:
     """
     Definition of the /transaction endpoint, in accordance with SEP-0006.
     See: https://github.com/stellar/stellar-protocol/blob/master/ecosystem/sep-0006.md#single-historical-transaction
